# -*- coding: utf-8 -*-

from django.apps import apps, AppConfig
from django.db.models.signals import post_migrate
from django.utils.encoding import force_str


class PortailUserConfig(AppConfig):
    name = "portailuser"
    label = "portailuser"
    verbose_name = "Gestion des utilisateurs du portail"

    def ready(self):
        post_migrate.connect(self._update_permissions, sender=self)

    @classmethod
    def _update_permissions(cls, sender, **kwargs):
        from .models import Ecole, Permission

        ecoles_pks = {}
        ecoles = {}
        for ecole in Ecole.objects.all():
            ecoles_pks[ecole.ident] = ecole.pk
            ecoles[ecole.pk] = ecole

        required_permissions = set()
        used_idents = set()

        for app in apps.get_app_configs():
            app_permissions = getattr(app, "app_permissions", [])

            for ident, name, desc, scope in app_permissions:
                ident = force_str(ident)
                name = force_str(name)
                desc = force_str(desc)

                assert ident not in used_idents, (
                    "Duplicate permission " "identifier " + ident + " (found in " + app + ")"
                )

                used_idents.add(ident)

                if scope == "globale":
                    required_permissions.add((ident, name, desc, None))

                elif scope == "chaque_ecole":
                    for ecole in ecoles_pks.values():
                        required_permissions.add((ident, name, desc, ecole))

                elif isinstance(scope, (list, tuple)):
                    for ecole in scope:
                        required_permissions.add((ident, name, desc, ecoles_pks[ecole]))

        current_permissions = set(Permission.objects.all().values_list("ident", "nom", "desc", "ecole"))

        to_create = required_permissions - current_permissions
        if to_create:
            for ident, name, desc, ecole_id in sorted(to_create):
                ecole = None if ecole_id is None else ecoles[ecole_id]
                print("Adding permission " + ident + " for " + (str(ecole) if ecole else "all"))
                permission = Permission(ident=ident, nom=name, desc=desc, ecole=ecole)
                super(Permission, permission).save()

        to_delete = current_permissions - required_permissions
        if to_delete:
            for ident, name, desc, ecole_id in sorted(to_delete):
                ecole = None if ecole_id is None else ecoles[ecole_id]
                print("Removing permission " + ident + " for " + (str(ecole) if ecole else "all"))
                permission = Permission.objects.get(ident=ident, nom=name, desc=desc, ecole=ecole)
                super(Permission, permission).delete()
